@@ -48,32 +48,31 @@ static char sccsid[] = "@(#)portmap.c 1.32 87/08/06 Copyr 1984 Sun Micro";
  */
 
 /*
- * Sun RPC is a product of Sun Microsystems, Inc. and is provided for
- * unrestricted use provided that this legend is included on all tape
- * media and as a part of the software program in whole or part.  Users
- * may copy or modify Sun RPC without charge, but are not authorized
- * to license or distribute it to anyone else except as part of a product or
- * program developed by the user.
- * 
- * SUN RPC IS PROVIDED AS IS WITH NO WARRANTIES OF ANY KIND INCLUDING THE
- * WARRANTIES OF DESIGN, MERCHANTIBILITY AND FITNESS FOR A PARTICULAR
- * PURPOSE, OR ARISING FROM A COURSE OF DEALING, USAGE OR TRADE PRACTICE.
- * 
- * Sun RPC is provided with no support and without any obligation on the
- * part of Sun Microsystems, Inc. to assist in its use, correction,
- * modification or enhancement.
- * 
- * SUN MICROSYSTEMS, INC. SHALL HAVE NO LIABILITY WITH RESPECT TO THE
- * INFRINGEMENT OF COPYRIGHTS, TRADE SECRETS OR ANY PATENTS BY SUN RPC
- * OR ANY PART THEREOF.
- * 
- * In no event will Sun Microsystems, Inc. be liable for any lost revenue
- * or profits or other special, indirect and consequential damages, even if
- * Sun has been advised of the possibility of such damages.
- * 
- * Sun Microsystems, Inc.
- * 2550 Garcia Avenue
- * Mountain View, California  94043
+ * Copyright (c) 2009, Sun Microsystems, Inc.
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions are met:
+ * - Redistributions of source code must retain the above copyright notice,
+ *   this list of conditions and the following disclaimer.
+ * - Redistributions in binary form must reproduce the above copyright notice,
+ *   this list of conditions and the following disclaimer in the documentation
+ *   and/or other materials provided with the distribution.
+ * - Neither the name of Sun Microsystems, Inc. nor the names of its
+ *   contributors may be used to endorse or promote products derived
+ *   from this software without specific prior written permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
+ * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+ * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+ * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
+ * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
+ * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
+ * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
+ * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
+ * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
+ * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
+ * POSSIBILITY OF SUCH DAMAGE.
  */
 
 #include <rpc/rpc.h>
@@ -99,7 +98,13 @@ static char sccsid[] = "@(#)portmap.c 1.32 87/08/06 Copyr 1984 Sun Micro";
 #include <stdlib.h>
 #include <pwd.h>
 
-#include "config.h"
+#ifndef LOG_PERROR
+#define LOG_PERROR 0
+#endif
+
+#ifndef LOG_DAEMON
+#define LOG_DAEMON 0
+#endif
 
 /* Older SYSV. */
 #if !defined(SIGCHLD) && defined(SIGCLD)
@@ -143,10 +148,13 @@ static int on = 1;
 #endif
 #endif
 
-int daemon_port = PMAPPORT;
+#ifdef DAEMON_UID
 int daemon_uid = DAEMON_UID;
 int daemon_gid = DAEMON_GID;
-const char* mapping_file = PORTMAP_MAPPING_FILE;
+#else
+int daemon_uid = 1;
+int daemon_gid = 1;
+#endif
 
 /*
  * We record with each registration a flag telling whether it was
@@ -160,38 +168,6 @@ struct flagged_pml {
 	int priv;
 };
 
-static inline int __getuid(const char* username)
-{
-	struct passwd* pw = getpwnam(username);
-
-	if (!pw)
-		return 0;
-
-	daemon_uid = pw->pw_uid;
-	daemon_gid = pw->pw_gid;
-	return 1;
-}
-
-static void usage(char *progname)
-{
-	fprintf(stderr,	"usage: %s [-dfFlv] [-t dir] [-i address] "
-			"[-u uid] [-g gid] [-U username] \n",
-		progname);
-	fprintf(stderr, "-v             verbose logging\n");
-	fprintf(stderr, "-d             debugging mode\n");
-	fprintf(stderr,	"-f             don't daemonize, log to standard error\n");
-	fprintf(stderr, "-F             don't daemonize, log as usual\n");
-	fprintf(stderr, "-t <dir>       chroot into dir\n");
-	fprintf(stderr, "-i <address>   bind to address\n");
-	fprintf(stderr, "-l             same as -i 127.0.0.1\n");
-	fprintf(stderr, "-u <uid>       run as this uid (default: %d)\n", DAEMON_UID);
-	fprintf(stderr, "-g <uid>       run as this gid (default: %d)\n", DAEMON_GID);
-	fprintf(stderr, "-p <port>      run on nonstandard port (default: %d)\n", daemon_port);
-	fprintf(stderr, "-U <username>  suid/sgid to this user\n");
-	fprintf(stderr, "-m <mapfile>   specify the mapping file name "
-					"(default: "PORTMAP_MAPPING_FILE")\n");
-}
-
 int
 main(int argc, char **argv)
 {
@@ -207,28 +183,13 @@ main(int argc, char **argv)
 	int foreground = 0;
 	int have_uid = 0;
 
-	while ((c = getopt(argc, argv, "hVdfFlt:vi:u:U:g:m:p:")) != EOF) {
+	while ((c = getopt(argc, argv, "Vdflt:vi:u:g:")) != EOF) {
 		switch (c) {
 
 		case 'V':
-			printf("portmap version 6.0.0.1 - 2008-05-10\n");
+			printf("portmap version 6.0 - 2007-May-11\n");
 			exit(1);
 
-		case 'm':
-			mapping_file = optarg;
-			break;
-
-		case 'U':
-			/* try to fetch user-given uid/gid by name */
-			if (!__getuid(optarg))
-			{
-				fprintf(stderr,
-					"portmap: illegal username: \"%s\"\n",
-					optarg);
-				exit(1);
-			}
-			have_uid = 1;
-			break;
 		case 'u':
 			daemon_uid = atoi(optarg);
 			if (daemon_uid <= 0) {
@@ -252,10 +213,6 @@ main(int argc, char **argv)
 		case 'f':
 			foreground = 1;
 			break;
-		case 'F':
-			/* run in foreground, but still log as usual */
-			foreground = 2;
-			break;
 
 		case 't':
 			chroot_path = optarg;
@@ -266,17 +223,25 @@ main(int argc, char **argv)
 			break;
 
 		case 'l':
-			optarg = (char*)"127.0.0.1";
+			optarg = "127.0.0.1";
 			/* FALL THROUGH */
 		case 'i':
 			have_bindaddr = inet_aton(optarg, &bindaddr);
 			break;
-		case 'p':
-			daemon_port = atoi(optarg);
-			break;
-		case 'h':
 		default:
-			usage(argv[0]);
+			fprintf(stderr,
+				"usage: %s [-dflv] [-t dir] [-i address] "
+				"[-u uid] [-g gid]\n",
+				argv[0]);
+			fprintf(stderr, "-d: debugging mode\n");
+			fprintf(stderr,
+				"-f: don't daemonize, log to standard error\n");
+			fprintf(stderr, "-t dir: chroot into dir\n");
+			fprintf(stderr, "-v: verbose logging\n");
+			fprintf(stderr, "-i address: bind to address\n");
+			fprintf(stderr, "-l: same as -i 127.0.0.1\n");
+			fprintf(stderr, "-u uid : setuid to this uid\n");
+			fprintf(stderr, "-g uid : setgid to this gid\n");
 			exit(1);
 		}
 	}
@@ -287,17 +252,20 @@ main(int argc, char **argv)
 	}
 
 #ifdef LOG_DAEMON
-	openlog("portmap",
-		LOG_PID|LOG_NDELAY | ( (foreground==1) ? LOG_PERROR : 0),
-		FACILITY);
+	openlog("portmap", LOG_PID|LOG_NDELAY | ( foreground ? LOG_PERROR : 0),
+	    FACILITY);
 #else
-	openlog("portmap",
-		LOG_PID|LOG_NDELAY | ( (foreground==1) ? LOG_PERROR : 0));
+	openlog("portmap", LOG_PID|LOG_NDELAY | ( foreground ? LOG_PERROR : 0));
 #endif
 
 #ifdef RPCUSER
 	if (!have_uid) {
-		if (!__getuid(RPCUSER))
+		struct passwd *pwent;
+		pwent = getpwnam(RPCUSER);
+		if (pwent) {
+			daemon_uid = pwent->pw_uid;
+			daemon_gid = pwent->pw_gid;
+		} else
 			syslog(LOG_WARNING, "user '" RPCUSER
 			       "' not found, reverting to default uid");
 	}
@@ -314,7 +282,7 @@ main(int argc, char **argv)
 	memset((char *) &addr, 0, sizeof(addr));
 	addr.sin_addr.s_addr = 0;
 	addr.sin_family = AF_INET;
-	addr.sin_port = htons(daemon_port);
+	addr.sin_port = htons(PMAPPORT);
 	if (have_bindaddr)
 		memcpy(&addr.sin_addr, &bindaddr, sizeof(bindaddr));
 
@@ -335,7 +303,7 @@ main(int argc, char **argv)
 	pml->pml_map.pm_prog = PMAPPROG;
 	pml->pml_map.pm_vers = PMAPVERS;
 	pml->pml_map.pm_prot = IPPROTO_UDP;
-	pml->pml_map.pm_port = daemon_port;
+	pml->pml_map.pm_port = PMAPPORT;
 	pmaplist = pml;
 
 	if ((sock = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP)) < 0) {
@@ -361,7 +329,7 @@ main(int argc, char **argv)
 	pml->pml_map.pm_prog = PMAPPROG;
 	pml->pml_map.pm_vers = PMAPVERS;
 	pml->pml_map.pm_prot = IPPROTO_TCP;
-	pml->pml_map.pm_port = daemon_port;
+	pml->pml_map.pm_port = PMAPPORT;
 	pml->pml_next = pmaplist;
 	pmaplist = pml;
 
@@ -400,7 +368,7 @@ main(int argc, char **argv)
 
 	(void)svc_register(xprt, PMAPPROG, PMAPVERS, reg_service, FALSE);
 
-	store_fd = open(mapping_file, O_RDWR|O_CREAT, PORTMAP_MAPPING_FMODE);
+	store_fd = open("/var/run/portmap_mapping", O_RDWR|O_CREAT, 0600);
 	load_table();
 
 	/* additional initializations */
@@ -422,14 +390,12 @@ main(int argc, char **argv)
 	abort();
 }
 
-#ifndef NO_PERROR
 /* need to override perror calls in rpc library */
 void perror(const char *what)
 {
 
 	syslog(LOG_ERR, "%s: %m", what);
 }
-#endif
 
 static struct pmaplist *
 find_service(u_long prog, u_long vers, u_long prot)
@@ -568,14 +534,17 @@ static void reg_service(struct svc_req *rqstp, SVCXPRT *xprt)
 				if (!check_privileged_port(svc_getcaller(xprt), 
 				    rqstp->rq_proc, 
 				    reg.pm_prog, 
-				    pml->pml_map.pm_port))
-					continue;
-
+				    pml->pml_map.pm_port)) {
+					ans = 0;
+					break;
+				}
 				fpml = (struct flagged_pml*)pml;
 				if (fpml->priv &&
 				    (ntohs(svc_getcaller(xprt)->sin_port)
-				     >= IPPORT_RESERVED))
-					continue;
+				     >= IPPORT_RESERVED)) {
+					ans = 0;
+					break;
+				}
 
 				ans = 1;
 				t = (caddr_t)pml;
@@ -783,7 +752,6 @@ static void callit(struct svc_req *rqstp, SVCXPRT *xprt)
 	if ((pml = find_service(a.rmt_prog, a.rmt_vers,
 	    (u_long)IPPROTO_UDP)) == NULL)
 		return;
-#ifndef NO_FORK
 	/*
 	 * fork a child to do the work.  Parent immediately returns.
 	 * Child exits upon completion.
@@ -794,7 +762,6 @@ static void callit(struct svc_req *rqstp, SVCXPRT *xprt)
 			    a.rmt_prog);
 		return;
 	}
-#endif
 	port = pml->pml_map.pm_port;
 	get_myaddress(&me);
 	me.sin_port = htons(port);
@@ -815,9 +782,7 @@ static void callit(struct svc_req *rqstp, SVCXPRT *xprt)
 		clnt_destroy(client);
 	}
 	(void)close(so);
-#ifndef NO_FORK
 	exit(0);
-#endif
 }
 
 #ifndef IGNORE_SIGCHLD			/* Lionel Cons <cons@dxcern.cern.ch> */
@@ -868,7 +833,7 @@ static void load_table(void)
 	struct flagged_pml fpml, *fpmlp;
 
 	ep = &pmaplist;
-	while (*ep)
+	while ((*ep)->pml_next)
 		ep = & (*ep)->pml_next;
 
 	if (store_fd < 0)
@@ -884,7 +849,7 @@ static void load_table(void)
 		      &fpml.pml.pml_map.pm_prot,
 		      &fpml.pml.pml_map.pm_port,
 		      &fpml.priv) == 5) {
-		if (fpml.pml.pml_map.pm_port == daemon_port)
+		if (fpml.pml.pml_map.pm_port == PMAPPORT)
 			continue;
 		fpmlp = malloc(sizeof(struct flagged_pml));
 		if (!fpmlp)
